' demo for CAN transmission with AT90CAN128 and BASCOM (Version 1.11.9.3)
'
' sender-hardware  : AT90CAN128 with PCA82C250, 16 MHz-Quarz and 2x 68pF
' receiver-hardware: AT90CAN128 with PCA82C250, 16 MHz-Quarz and 2x 68pF, MAX232 at PE0 and PE1 for serial connection to PC
'
' the sender sends messages to id 0 up to id 100, even id's with 11 bit identifier, odd id's with 29 bit identifier
' use CAN_TX to send a message, you can choose every MOb from 0 to 14, you can use more than 1 MOb to send a message
' after transmission the MOb is deactivated and have to "restart" by a new trasnmission via CAN_TX
'
' the receiver software contains different demos, to show how the filter works (see below)
' use CAN_RX to receive a message, you can choose every MOb from 0 to 14, you can use more than 1 MOb to receive a message
' after receiving the MOb is deactivated and is "restarted" by CAN_RX
' the interrupts are activated, but i don't use them, CAN_RX works with the polling method
'
' some parts of my code are taken from Sam Berglund, sorry Sam, your homepage is broken,
' i couldn't ask for permission, but thank you for inspiration
' http://web.archive.org/web/20070428165622/http://www.samberglund.se/AVR/sb_can_m128.bas
'
' this code is free for non-commercial use

$regfile = "m128can.dat"                                    ' AT90CAN128
$crystal = 16000000                                         ' 16 MHz
$prog &HFF , &HFF , &HD9 , &HFF                             ' 8-16 MHz, ext. Quarz
$baud = 9600
$hwstack = 100
$swstack = 130
$framesize = 100

Const Can_11bit = 0                                         ' length of identifier
Const Can_29bit = 1                                         ' length of identifier

Declare Sub Can_init(byval Bitrate As Byte)                 ' init, bitrate with 16 MHz-Quarz
Declare Sub Can_reset_mob(byval Mob As Byte)                ' reset MOb x
Declare Sub Can_reset_all_mob()                             ' reset all MOb's
Declare Sub Can_rx_mob(byval Mob As Byte)                   ' define RX-MOb
Declare Sub Can_rx()                                        ' poll to receive data
Declare Sub Can_tx(byval Mob As Byte , Byval Can_tx_id As Long , Byval Can_tx_dlc As Byte , Byval Bitmode As Byte)       ' create send-MOb and send data
Declare Sub Can_set_mask(byval Mob As Byte , Byval Rx_id As Long , Byval Mask As Long , Byval Bitmode As Byte)       ' set filter
Declare Function Can_mob_used(byval Mob As Byte) As Byte    ' =1, if MOb used
Declare Function Can_transmitted(byval Mob As Byte) As Byte ' =1, if message transmitted
Declare Function Can_error_passive() As Byte                ' =1, if error passive mode
Declare Function Can_bus_off() As Byte                      ' =1, if bus off mode

Dim Can_tx_data(8) As Byte                                  ' contains data to send
Dim Can_rx_data(8) As Byte                                  ' contains received data
Dim Can_rx_id As Long                                       ' receiver id
Dim Can_rx_dlc As Byte                                      ' length of datafield
Dim Can_rx_bitmode As Byte                                  ' 11 or 29 bit identifier

Dim Id_to_send As Long
Dim Even_value As Byte
Dim Count As Byte

'----------------------------------------------------------------------------

  Enable Interrupts
  Call Can_init(0)
  Print
  Print "start"
  Id_to_send = 0
  Do
    Can_tx_data(1) = Id_to_send                             ' fill datafield 1 with id-number
    Even_value = Id_to_send And 1
    If Even_value = 0 Then
      Call Can_tx(8 , Id_to_send , 1 , Can_11bit)           ' send via MOb 8, to id, 1 databyte, even id as 11 bit mode
    Else
      Call Can_tx(8 , Id_to_send , 1 , Can_29bit)           ' send via MOb 8, to id, 1 databyte, odd id as 29 bit mode
    End If
    Incr Id_to_send
    If Id_to_send > 100 Then Id_to_send = 0
    Wait 1
  Loop
End


' CAN-Prozeduren ---------------------------------------------------------------

Sub Can_init(byval Bitrate As Byte)
  Cangie = &HFE                                             ' enable interrupt
  Canie1 = &H7F                                             ' enable interrupt
  Canie2 = &HFF                                             ' enable interrupt
  Select Case Bitrate
    Case 0 : Canbt1 = &H00                                  ' 1 MBit/s
    Case 1 : Canbt1 = &H02                                  ' 500 KBit/s
    Case 2 : Canbt1 = &H06                                  ' 250 KBit/s
    Case 3 : Canbt1 = &H08                                  ' 200 KBit/s
    Case 4 : Canbt1 = &H0E                                  ' 125 KBit/s
    Case 5 : Canbt1 = &H12                                  ' 100 KBit/s
  End Select
  Canbt2 = &H0C                                             ' bitrate, change if Quarz<>16 MHz
  Canbt3 = &H37                                             ' bitrate, change if Quarz<>16 MHz
  Cangcon = &H02                                            ' enable CAN-channel
  Call Can_reset_all_mob                                    ' reset all MOb
End Sub


Sub Can_reset_mob(byval Mob As Byte)

  Local Mob_count As Byte

  Shift Mob , Left , 4
  Canpage = Mob                                             ' select MOb
  Canstmob = 0                                              ' clear  status register
  Cancdmob = 0                                              ' clear control register
  Canidt4 = 0                                               ' clear ID register
  Canidt3 = 0
  Canidt2 = 0
  Canidt1 = 0
  Canidm4 = 0                                               ' clear mask register
  Canidm3 = 0
  Canidm2 = 0
  Canidm1 = 0
  For Mob_count = 0 To 7
    Canmsg = 0                                              ' clear message register
  Next
End Sub


Sub Can_reset_all_mob()

  Local Mob_count As Byte

  For Mob_count = 0 To 14
    Call Can_reset_mob(mob_count)
  Next
End Sub


Sub Can_rx_mob(byval Mob As Byte)
  Shift Mob , Left , 4
  Canpage = Mob                                             ' select MOb
  Canstmob = 0                                              ' clear  status register
  Cancdmob = 0                                              ' clear control register
  Canidt4 = 0                                               ' clear ID register
  Canidt3 = 0
  Canidt2 = 0
  Canidt1 = 0
  Canidm4 = 0                                               ' clear mask register
  Canidm3 = 0
  Canidm2 = 0
  Canidm1 = 0
  Cancdmob = &HC0                                           ' 11000000 = enable frame buffer reception
End Sub


Sub Can_rx()

  Local Adress As Word
  Local Data_count As Byte
  Local Mob_count As Byte
  Local Mob As Byte
  Local Received As Byte

  For Mob_count = 0 To 14
    Mob = Mob_count
    Shift Mob , Left , 4
    Canpage = Mob                                           ' select MOb
    Received = Canstmob And &B00100000                      ' bit 5 = RXOK
    If Received <> 0 Then
      Can_rx_dlc = Cancdmob And &H0F                        ' get length of datafield, DLC 0-3
      Can_rx_bitmode = Cancdmob And &H10                    ' get received identifier length 11 or 29 bit
      If Can_rx_bitmode = Can_11bit Then                    ' if 11 bit identifier
        Adress = Varptr(can_rx_id)                          ' get received 11 bit id
        Out Adress , Canidt2
        Incr Adress
        Out Adress , Canidt1
        Shift Can_rx_id , Right , 5
      Else                                                  ' if 29 bit identifier
        Adress = Varptr(can_rx_id)                          ' get received 29 bit id
        Out Adress , Canidt4
        Incr Adress
        Out Adress , Canidt3
        Incr Adress
        Out Adress , Canidt2
        Incr Adress
        Out Adress , Canidt1
        Shift Can_rx_id , Right , 3
      End If
      For Data_count = 1 To Can_rx_dlc
        Can_rx_data(data_count) = Canmsg                    ' get data
      Next
      Canstmob = &H00                                       ' reset status register
      If Can_rx_bitmode = Can_11bit Then
        Cancdmob = &HC0                                     ' restart MOb
      Else
        Cancdmob = &HD0                                     ' restart MOb + IDE, otherwise the filter doesn't work
      End If

      ' result :
      ' CAN_RX_ID       = received id
      ' CAN_RX_DLC      = length of datafield
      ' CAN_RX_Data 1-8 = data
      ' CAN_RX_Bitmode  = 11 or 29 bit

      ' **** here is your code ***


      Print
      Print "mob " ; Str(mob_count) ; "    from id " ; Str(can_rx_id)
      Print "data : " ;
      For Count = 1 To Can_rx_dlc
       Print Can_rx_data(count) ; " - ";
      Next
      If Can_rx_bitmode = Can_11bit Then
        Print "11 bit mode"
      Else
        Print "29 bit mode"
      End If


    End If
  Next
End Sub


Sub Can_tx(byval Mob As Byte , Byval Can_tx_id As Long , Byval Can_tx_dlc As Byte , Byval Bitmode As Byte)

  Local Adress As Word
  Local Data_count As Byte

  Shift Mob , Left , 4
  Canpage = Mob                                             ' select MOb
  Canstmob = 0                                              ' clear status register
  Cancdmob = 0                                              ' clear control register
  If Bitmode = Can_11bit Then                               ' if 11 bit identifier
    Shift Can_tx_id , Left , 5                              ' set 11 bit id
    Adress = Varptr(can_tx_id)
    Canidt4 = 0
    Canidt3 = 0
    Canidt2 = Inp(adress)
    Incr Adress
    Canidt1 = Inp(adress)
  Else                                                      ' 29 bit identifier
    Shift Can_tx_id , Left , 3                              ' set 29 bit id
    Adress = Varptr(can_tx_id)
    Canidt4 = Inp(adress)
    Incr Adress
    Canidt3 = Inp(adress)
    Incr Adress
    Canidt2 = Inp(adress)
    Incr Adress
    Canidt1 = Inp(adress)
  End If
  For Data_count = 1 To 8
    Canmsg = Can_tx_data(data_count)                        ' set data
  Next
  Cangit = &H80                                             ' enable interrupt
  If Bitmode = Can_11bit Then
    Cancdmob = Can_tx_dlc + &H40                            ' set transmission enable with 11 bit id
  Else
    Cancdmob = Can_tx_dlc + &H50                            ' set transmission enable with 29 bit id
  End If
  Cangie = &HFE                                             ' enable interrupt
  Canie1 = &H7F                                             ' enable interrupt
  Canie2 = &HFF                                             ' enable interrupt
End Sub


Sub Can_set_mask(byval Mob As Byte , Byval Rx_id As Long , Byval Mask As Long , Byval Bitmode As Byte)

  Local Adress As Word

  Shift Mob , Left , 4
  Canpage = Mob                                             ' select MOb
  If Bitmode = Can_11bit Then                               ' if 11 bit identifier
    Shift Rx_id , Left , 5                                  ' set 11 bit id
    Adress = Varptr(rx_id)
    Canidt4 = 0
    Canidt3 = 0
    Canidt2 = Inp(adress)
    Incr Adress
    Canidt1 = Inp(adress)
    Shift Mask , Left , 5                                   ' set 11 bit mask
    Adress = Varptr(mask)
    Canidm4 = 1
    Canidm3 = 0
    Canidm2 = Inp(adress)
    Incr Adress
    Canidm1 = Inp(adress)
  Else                                                      ' if 29 bit identifier
    Shift Rx_id , Left , 3                                  ' set 29 bit id
    Adress = Varptr(rx_id)
    Canidt4 = Inp(adress)
    Incr Adress
    Canidt3 = Inp(adress)
    Incr Adress
    Canidt2 = Inp(adress)
    Incr Adress
    Canidt1 = Inp(adress)
    Shift Mask , Left , 3                                   ' set 29 bit mask
    Adress = Varptr(mask)
    Canidm4 = Inp(adress) Or 1
    Incr Adress
    Canidm3 = Inp(adress)
    Incr Adress
    Canidm2 = Inp(adress)
    Incr Adress
    Canidm1 = Inp(adress)
    Cancdmob = Cancdmob Or &H10                             ' set IDE, otherwise the filter doesn't work
  End If
End Sub


Function Can_mob_used(byval Mob As Byte) As Byte            ' =1, if MOb used

  Local Bit_value As Word
  Local Mob_count As Byte
  Local Bit_count As Word
  Local Dummy As Word

  Dummy = Canen1
  Shift Dummy , Left , 8
  Dummy = Dummy + Canen2
  Bit_count = 1
  For Mob_count = 0 To Mob
    If Mob_count = Mob Then
      Bit_value = Dummy And Bit_count
      If Bit_value <> 0 Then
        Can_mob_used = 1
      Else
        Can_mob_used = 0
      End If
    End If
    Bit_count = Bit_count * 2
  Next
End Function


Function Can_transmitted(byval Mob As Byte) As Byte         ' =1, if message transmitted

  Local Transmitted As Byte

  Shift Mob , Left , 4
  Canpage = Mob
  Transmitted = Canstmob And &B01000000
  If Transmitted <> 0 Then
    Can_transmitted = 1
  Else
    Can_transmitted = 0
  End If
End Function


Function Can_error_passive() As Byte                        ' =1, if error passive mode

  Local Error_passive As Byte

  Error_passive = Cangsta And &B00000001
  If Error_passive <> 0 Then
    Can_error_passive = 1
  Else
    Can_error_passive = 0
  End If
End Function


Function Can_bus_off() As Byte                              ' =1, if bus off mode

  Local Bus_off As Byte

  Bus_off = Cangsta And &B00000010
  If Bus_off <> 0 Then
    Can_bus_off = 1
  Else
    Can_bus_off = 0
  End If
End Function
